#!/bin/sh

###################################################################
#Written by: Truc Nguyen 2001
# Name   : emailstats.sh
# Purpose: This shell is used to send the imager ASOS 
#          error message to programmer.
# Called by:  geimager.mac and gwimager.mac
# command: emailnote_LI.sh $1 where $1 is CSTAT
# Input:
#     CSTAT such as "ERRSECL" or "ERRAREA" from SFOV processing.
# updated by: Truc Nguyen 01/23/2014 for Sateps Migration CCR#2126
###################################################################

SCRIPTDIR=$( dirname $0 )
# call env setup file
. $HOME/iasos/scripts/iasos_env.sh

export tstrb=`date +%H%M%S`

script_name=`basename $0`
echo $script_name


TIME=`date +%H0000`
DAY=`date +%Y%j`
Subject="IMAGER ASOS STATUS REPORT"
mach=`uname -n`

cd $iasos_data
if [ $1 == "ERRSECL" ]
then
    echo "****If error is ERRSECL; check for missing images****" > STATSFILE
    imglist.k LOC/AREA.-4 DEV=T STATSFILE A
fi

if [ $1 == "ERRFECL" ]
then
    echo "****If error is ERRFECL; check for missing images****" > STATSFILE
    imglist.k LOC/AREA.-4 DEV=T STATSFILE A
fi

if [ $1 == "ERRAREA" ]
then
    echo "****If error is ERRAREA; check for missing images****" > STATSFILE
    imglist.k LOC/AREA.-4 DEV=T STATSFILE A
fi

if [ $1 == "ERRCHAN" ]
then
    echo "****If error is ERRCHAN; check for missing bands 3 & 4****" > STATSFILE
    imglist.k LOC/AREA.-4 DEV=T STATSFILE A
fi

if [ $1 == "ERRNSFC" ]
then
    echo "***If error is ERRNSFC check for current DAY and TIME" > STATSFILE
    echo "   in the file below***"                               >> STATSFILE
    echo "***The current DAY= $DAY  The current TIME= $TIME"     >> STATSFILE
    echo " "                                                     >> STATSFILE
    cat MdoxResult                                               >> STATSFILE
fi

if [ $1 == "ERROLIM" ]
then
    echo "***The ERROLIM status indicates the imagery for this day "  > STATSFILE
    echo "   and time has already been processed.***"                >> STATSFILE
fi 

if [ $1 == "ERRGRID" ]
then
    echo "***The ERRGRID status indicates the grids for this run "  > STATSFILE
    echo "   were not processed correctly. ***"                    >> STATSFILE
fi

if [ $1 == "ERRVAPR" ]
then
    echo "***The ERRVAPR indicates PROBLEM OPENING/READING VASTEXT FILE " > STATSFILE
fi

#if [ $1 == "SUCCESS" ]
#    then
#    echo "EAST IMAGER ASOS"      > STATSFILE
#    echo "no action required "  >> STATSFILE
#    tail -7 ASOSTATS            >> STATSFILE
#    cat Clienttemp_nws          >> STATSFILE
#fi

if [ -f STATSFILE ]
then
 
   echo " "         >> STATSFILE
   tail -4 ASOSTATS >> STATSFILE

   if [ -f emaillistFile ]
   then
      exec < emaillist
      read EmailAddress
      while [ $EmailAddress ]
         do
                  mail -s "$Subject" $EmailAddress < STATSFILE
            read EmailAddress
          done
   fi
fi

mv $iasos_data/STATSFILE $iasos_log

exit
