#!/bin/bash

export local_mcidas=$HOME/mcidas
export local_bin=$local_mcidas/bin
export local_data=$local_mcidas/data

export iasos_home=$HOME/iasos
export iasos_bin=$iasos_home/bin
export iasos_scripts=$iasos_home/scripts
export iasos_data=$iasos_home/data


export app_data_dir=/data/Piasos
#export iasos_data=$app_data_dir/data
export iasos_area=$app_data_dir/area
export iasos_md=$app_data_dir/md
export iasos_grid=$app_data_dir/grid
export iasos_log=$app_data_dir/logs
export iasos_archive=$app_data_dir/archive

export mcidas_home=/home/mcidas
export mcidas_bin=$mcidas_home/bin

export plocal=/home/Plocal

#export SFS_status_dir=/data/Plocal/SFS/ops/prod/status_files
#export SFS_iasos_prod_dir=/data/Plocal/SFS/ops/prod/shared/iasos

#export SFS_status_dir=/gpfs1/ops/prod/status_files
#export SFS_iasos_prod_dir=/gpfs1/ops/prod/shared/iasos

export SFS_status_dir=/gpfs1/ops/test/status_files
export SFS_iasos_prod_dir=/gpfs1/ops/test/shared/iasos


export  PATH=$PATH:.:$iasos_home:$iasos_bin:$HOME/mcidas/bin:$mcidas_home/bin
export  MCPATH=$HOME/mcidas/bin:$mcidas_home/bin:$HOME/mcidas/data:$mcidas_home/data
export  MCDATA=$iasos_area

export PATH=$PATH:/usr/kerberos/bin:/usr/local/bin:/bin:/usr/bin:/usr/X11R6/bin:~mcidas/bin:$HOME/mcidas/bin:$HOME/mcidas/data
export MCPATH=~mcidas/data:$HOME/mcidas/data:~mcidas/help:$HOME/mcidas/help


export recipient="truc.nguyen@noaa.gov truc.nguyen@gmail.com"
export DDShost="dds.nesdis.noaa.gov"
export  SVGAXA=$plocal/bin/svgaxa.k
export appname="IASOS"

#===============================================================================

export DDS_DISTRIBUTION=NO

#send monitor products to sandia
#export PRODUCT_MONITOR=NO
export PRODUCT_MONITOR=YES
monitoring_machine_dir="Psar@10.144.3.35:/home/Psar/Piasos/status"
monitoring_log_dir="Psar@10.144.3.35:/home/Psar/Piasos/logs"

#set up warning mail messages
email_list="truc.nguyen@noaa.gov"

#
# Define a function to transfer status files to the monitoring machine.
# Use "PasswordAuthentication=no" so the command does not ask for a password
# which will cause the script to wait for a password to be entered.  We assume
# a private/public key combination has been set up between these two machines
# which allows for passwordless scp execution.
#

if [[ $PRODUCT_MONITOR = "YES" ]]
then

transfer_status_file () {
   echo " "
#   echo " transfer status file to $monitoring_machine_dir"
#   echo "scp -pq $1 $monitoring_machine_dir/."
#   scp -pq $1 $monitoring_machine_dir/.
#   check_status X "status file $1 transfer failed" 

   echo " transfer status file to $SFS_status_dir"
   echo "cp -p $1 $SFS_status_dir/."
   cp -p $1 $SFS_status_dir/.
   check_status X "status file $1 transfer failed" 

#   echo " transfer log file to $monitoring_log_dir"
#   echo "scp -pq $2 $monitoring_log_dir/."
#   scp -pq $2 $monitoring_log_dir/.
#   check_status X "log file $2 transfer failed" 
}

else

transfer_status_file () {
   echo 'Non-production machine, no transfer' 
}

fi

#
# Define a function to check the status of the last job and deliver 
# appropriate messages 
# Usage     
#  check_status <code> "<message>"
#
# Code can be
#  S       successful completion; program ends and sends log/status files
#  E       (default) critical error;  program ends and sends log/status files
#  F       force quit, like E, even if $? is good
#  N       no data found - success, but check # since last good run
#  X       warning message; program continues
#  W       warning message; program ends and sends log/status files
#
#  The message, quoted, can be just about anything
#
#  Note that start_date, status_file, and log_file must be defined in the main program

check_status () 
{

#set the $? variable immediately
   program_status=$?
   echo "PGM_STATUS=<$program_status>"

#check the arguments, supply defaults
   if [ $# -ge 2 ]
   then
     error_status_code=$1
     error_message=$2
   elif [ $# -eq 1 ]
   then
     error_status_code=$1
     error_message='Error in program'
   else
     error_status_code='E'
     error_message='Error in program'
   fi

   status_time=$(date -u +%Y%m%d%H%M%S)

   case $error_status_code in

   X)       # check for an error that is NOT FATAL and the program should continue
     if [[ $program_status -ne 0 ]]
     then
       echo ^CHECK_STATUS WARNING At $status_time $error_message
     fi
     return ;;

   W)       # to send a WARNING message and end the program
     echo At $status_time  $error_status_code $error_message
     echo $start_date $status_time $error_status_code $log_file detected warning msg. in log file >> $status_file
     transfer_status_file $status_file $log_file
     return 0 ;;

   S)       # if given a successful completion code
     echo At $status_time  $error_status_code $error_message
     echo $start_date $status_time $error_status_code $log_file successful completion >> $status_file
     transfer_status_file $status_file $log_file
     return 0 ;;
   N)      # no data found; conditional success, check for how long since good run
     if [ $# -ge 3 ]
     then
       check_num=$3
     else
       check_num=10
     fi
     success_num=$(tail -$check_num $status_file|grep " successful completion"|wc -l)
     if [ $success_num -gt 0 ]
     then
       echo At $status_time N $error_message
       echo $start_date $status_time N $log_file No new data found >> $status_file
       transfer_status_file $status_file $log_file
       return 0
     else
       echo At $status_time E Too many runs since last success
       echo $start_date $status_time E $log_file Too many runs since last success >> $status_file
       transfer_status_file $status_file $log_file
       tail -1 $status_file | mail -s "$appname Failure" $email_list
       return 99
     fi ;;

   F)      # if a fatal error that should stop the program
     echo At $status_time E $error_message 
     echo $start_date $status_time E $log_file $error_message  >> $status_file
     transfer_status_file $status_file $log_file
     tail -1 $status_file | mail -s "$appname Failure" $email_list
     exit 99 ;;


   E)    # if status is bad AND it is a fatal error that should stop the program
     if [[ $program_status -ne 0 ]]
     then
      echo At $status_time  $error_status_code $error_message 
      echo $start_date $status_time $error_status_code $log_file program failed >> $status_file
      transfer_status_file $status_file $log_file
      tail -1 $status_file | mail -s "$appname Failure" $email_list
      exit 99
    fi ;;

   *)    #bad code entry
      echo At $status_time  $error_status_code "Check_status fail - bad status code"
      echo $start_date $status_time $error_status_code $log_file check_status failed >> $status_file
      transfer_status_file $status_file $log_file
      tail -1 $status_file | mail -s "$appname Failure" $email_list
      exit 99 ;;

   esac
}

