      SUBROUTINE UPRAOB(MD,IDAY,TIME,IDN,NLEV,ISTDAT,
     * LEV,DP,DT,DTD,DIR,DSP,DZ,ISTAT)
C $ SUBROUTINE UPRAOB(MD, IDAY, TIME, IDN, NLEV, ISTDAT,     (BTR)
C $   *  LEV, DP, DT, DTD, DIR, DSP, DZ, ISTAT)
C $ RETRIEVE RAOB DATA FROM MD FILE WITH SCHEMA THAT MEETS FOLLOWING
C $   CONDITIONS:  1) KEYS 'DAY' AND 'TIME' APPEAR IN ROW HEADER, &
C $   ONE OF THEM IS THE FIRST KEY IN HEADER
C $   2) KEYS "ID" OR "IDN", AND/OR "LAT" AND "LON" APPEAR IN COLUMN
C $   HEADER OR DATA SECTION (IF NO COLUMN HEADERS). IF COLUMN HEADER
C $   IS PART OF SCHEMA, ONE OF THESE MUST BE FIRST KEY IN HEADER.
C $   3) AT LEAST ONE OF THE FOLLOWING KEYS MUST APPEAR IN THE DATA
C $   SECTION OF THE RECORD: P, T, TD, DIR, SPD, Z, LEV.
C $   RETURNS 0 (DATA OK), -1 (NO DATA) OR -2 (INVALID SCHEMA).
C $ INPUT:
C $   MD = (I) MD FILE #
C $   IDAY = (I) DAY OF DATA (YYDDD)
C $   TIME = (I) DATA TIME (HHMMSS)
C $   IDN = (I) STATION IDENTIFIER
C $ OUTPUT:
C $   NLEV = (I) NUMBER OF LEVELS (MAX=100)
C $   ISTDAT = (I) 4-WORD ARRAY TO CONTAIN LAT, LON, ELEVATION, &
C $      STATE CODE OF DESIRED STATION
C $   LEV = (I) ARRAY CONTAINING PRESSURE LEVELS (MB)
C $   DP = (R) ARRAY CONTAINING PRESSURE (MB)
C $   DT = (R) ARRAY CONTAINING TEMP (DEG K)
C $   DTD = (R) ARRAY CONTAINING DEWPOINT (DEG K)
C $   DIR = (R) ARRAY CONTAINING WIND DIRECTION (DEG)
C $   DSP = (R) ARRAY CONTAINING WIND SPEED (MPS)
C $   DZ = (R) ARRAY CONTAINING HEIGHT (M)
C $   ISTAT = (I) STATUS CODE: 0 (OK; BUT SOME DATA MAY BE MISSING)
C $      -1 (DATA MISSING; NO SOUNDING RETURNED) -2 (NO MD FILE)
C $$ UPRAOB = OBSERVATION, MD, SCHEMA
C
      IMPLICIT INTEGER (A-B,E-Z)
      IMPLICIT CHARACTER*12 (C)
      COMMON/ANLFLG/NDP,NDT,NDTD,NDIR,NDSP,NDZ
      DIMENSION DP(*),DT(*),DTD(*),DIR(*),DSP(*),DZ(*),ISTDAT(*),LEV(*)
      DIMENSION MDHD(64),SCALE(400),LIST(400),UNIT(400),LOC(400)
      DIMENSION RECORD(400)
C      DATA MDFLG/0/,IDYLOC/0/,TIMLOC/0/,IDLOC/0/,LATLOC/0/,LONLOC/0/
C      DATA STLOC/0/,ZSLOC/0/,PLOC/0/,TLOC/0/,ZLOC/0/,TDLOC/0/,LEVLOC/0/
C      DATA DIRLOC/0/,SPDLOC/0/,MAXLOC/0/
C-----NR,NC ARE THE # OF ROWS & COLUMNS IN THIS FILE
      EQUIVALENCE (NR,MDHD(4)),(NC,MDHD(5))
C-----NKEYS IS THE TOTAL # OF RECORD KEYS (= RECORD LENGTH, IN WORDS)
C-----NRKEYS,NCKEYS,NDKEYS GIVE THE # OF KEYS IN THE ROW HEADER, COL
C-----   HEADER, AND DATA PORTION, RESP. SUM MUST BE = NKEYS
      EQUIVALENCE (NKEYS,MDHD(6)),(NRKEYS,MDHD(7)),
     *   (NCKEYS,MDHD(8)),(NDKEYS,MDHD(9))
C-----CPOS,DPOS GIVES (1-BASED) POS IN RECORD OF COL HDR, DATA PORTION
      EQUIVALENCE (NCPOS,MDHD(10)),(NDPOS,MDHD(11))
C-----NREPS,REPSIZ,REPPOS GIVES # OF REPEAT GROUPS (NULL CASE IS 1),
C-----   SIZE OF REP. GP (NULL CASE =NDKEYS) AND START OF GP (NULL=DPOS)
      EQUIVALENCE (NREPS,MDHD(12)),(REPSIZ,MDHD(13)),(REPPOS,MDHD(14))
C-----MISSING DATA CODE (BY 7/82 CONVENTION = Z80808080)
      DATA MDFLG/0/,IDYLOC/0/,TIMLOC/0/,IDLOC/0/,LATLOC/0/,LONLOC/0/
      DATA STLOC/0/,ZSLOC/0/,PLOC/0/,TLOC/0/,ZLOC/0/,TDLOC/0/,LEVLOC/0/
      DATA DIRLOC/0/,SPDLOC/0/,MAXLOC/0/
      DATA MDATA/Z'80808080'/
C
C---CHECK FOR INITIAL CALL TO UPRAOB
C
      IF(MDFLG.EQ.0) THEN
         ISTAT=-2
         IF(MDOPEN(MD,1).NE.0) RETURN
         IF(MDKEYS(MD,-1,LIST,SCALE,UNIT,LOC).EQ.-1) RETURN
         IF(MDINFO(MD,MDHD).NE.0) RETURN
C
C---CHECK FOR "DAY" AND "TIME" IN ROW HEADER
C
         DO 10 I=1,NRKEYS
         IF(LIST(I).EQ.LIT('DAY ')) IDYLOC=LOC(I)
         IF(LIST(I).EQ.LIT('TIME')) TIMLOC=LOC(I)
         IF(LIST(I).EQ.LIT('CMAX')) MAXLOC=LOC(I)
10       CONTINUE
      CALL DDEST('IDYLOC=',IDYLOC)
      CALL DDEST('TIMLOC=',TIMLOC)
         IF(IDYLOC.EQ.0.OR.TIMLOC.EQ.0) RETURN
C
C---CHECK FOR "ID" OR "IDN", OR "LAT" AND "LON" KEYS
C
         DO 20 I=NCPOS,NKEYS
         IF(LIST(I).EQ.LIT('ID  ').OR.LIST(I).EQ.LIT('IDN '))
     *   IDLOC=LOC(I)
         IF(LIST(I).EQ.LIT('LAT ')) LATLOC=LOC(I)
         IF(LIST(I).EQ.LIT('LON ')) LONLOC=LOC(I)
         IF(LIST(I).EQ.LIT('ZS  ').OR.LIST(I).EQ.LIT('Z0  '))
     *   ZSLOC=LOC(I)
         IF(LIST(I).EQ.LIT('ST  ').OR.LIST(I).EQ.LIT('LOC '))
     *   STLOC=LOC(I)
20       CONTINUE
      CALL DDEST('IDLOC=',IDLOC)
      CALL DDEST('LATLOC=',LATLOC)
      CALL DDEST('LONLOC=',LONLOC)
         IF(IDLOC.EQ.0.AND.(LATLOC.EQ.0.OR.LONLOC.EQ.0)) RETURN
C
C---CHECK FOR ARRAY KEYS "P","T","TD","SPD","DIR",AND/OR "DIR"
C
         EREP=REPPOS+REPSIZ-1
         DO 30 I=REPPOS,EREP
         IF(LIST(I).EQ.LIT('P   ')) PLOC=LOC(I)
         IF(LIST(I).EQ.LIT('T   ')) TLOC=LOC(I)
         IF(LIST(I).EQ.LIT('TD  ')) TDLOC=LOC(I)
         IF(LIST(I).EQ.LIT('SPD ').OR.LIST(I).EQ.LIT('UC  '))
     *   SPDLOC=LOC(I)
         IF(LIST(I).EQ.LIT('DIR ').OR.LIST(I).EQ.LIT('VC  '))
     *   IDRLOC=LOC(I)
         IF(LIST(I).EQ.LIT('LEV ')) LEVLOC=LOC(I)
         IF(LIST(I).EQ.LIT('Z   ')) ZLOC=LOC(I)
30       CONTINUE
      CALL DDEST('SPDLOC=',SPDLOC)
      CALL DDEST('IDRLOC=',IDRLOC)
      CALL DDEST('ZLOC=',ZLOC)
         ALLOC=PLOC+TLOC+TDLOC+SPDLOC+IDRLOC+LEVLOC+ZLOC
      CALL DDEST('ALLOC=',ALLOC)
         IF(ALLOC.EQ.0) RETURN
         MDFLG=1
      ENDIF
      ISTAT=-1
         LEVL=LEVLOC
         PL=PLOC
         TL=TLOC
         TDL=TDLOC
         ZL=ZLOC
         IDRL=IDRLOC
         SPDL=SPDLOC
C
C---FIND ROW CONTAINING THE DATA
C
         RST=1
         RED=NR
         ROW=IKWP('ROW',1,0)
         IF (ROW.NE.0) THEN
            RST=ROW
            RED=ROW
         ENDIF
         DO 40 I=RST,RED
         IF(MDGET(MD,I,0,RECORD).EQ.0) THEN
            IF(RECORD(IDYLOC).EQ.IDAY.AND.RECORD(TIMLOC).EQ.TIME) THEN
               ROW=I
               IF(MAXLOC.NE.0) MAX=RECORD(MAXLOC)
               GO TO 45
            ENDIF
         ENDIF
40       CONTINUE
         CALL EDEST('DAY TIME NOT FOUND',0)
         RETURN
C
C---FIND COLUMN CONTAINING THE DATA
C
45    NCOL=0
      END=NC
      IF(MAXLOC.NE.0) END=MAX
      CALL DDEST('NC=',NC)
      CALL DDEST('MAXLOC=',MAXLOC)
      CALL DDEST('END=',END)
      DO 50 I=1,END
      IF(MAXLOC.NE.0) THEN
         FNC=MDGET(MD,ROW,I,RECORD)
      ELSE
         FNC=MDGET(MD,0,I,RECORD)
      ENDIF
      IF(FNC.EQ.0) THEN
         IF(IDLOC.NE.0) THEN
            IF(RECORD(IDLOC).EQ.IDN) THEN
               NCOL=I
               CALL DDEST(' REPORT IS IN COLUMN ',NCOL)
               GO TO 55
            ENDIF
         ENDIF
      ENDIF
50    CONTINUE
      RETURN
C
C---GET THE DATA
C
55    IF(MDGET(MD,ROW,NCOL,RECORD).NE.0) THEN
         CALL EDEST('UNABLE TO RETRIEVE RECORD',0)
         RETURN
      ENDIF
      ISTDAT(1)=RECORD(LATLOC)
      ISTDAT(2)=RECORD(LONLOC)
      ISTDAT(3)=RECORD(ZSLOC)
      ISTDAT(4)=RECORD(STLOC)
C
C---FILL LEVEL GROUPS
C
      MLEV=0
      NDP=0
      NDT=0
      NDTD=0
      NDIR=0
      NDSP=0
      NDZ=0
      CALL DDEST('NREPS=',NREPS)
      DO 70 I=1,NREPS
      IF(RECORD(LEVL).NE.MDATA) THEN
         MLEV=MLEV+1
         LEV(MLEV)=RECORD(LEVL)
      ENDIF
      LEVL=LEVL+REPSIZ
      IF(RECORD(PL).NE.MDATA) THEN
         NDP=NDP+1
         DP(NDP)=FLOAT(RECORD(PL))/(10**SCALE(PLOC))
      ENDIF
      PL=PL+REPSIZ
      IF(RECORD(TL).NE.MDATA) THEN
         NDT=NDT+1
         DT(NDT)=FLOAT(RECORD(TL))/(10**SCALE(TLOC))
      ENDIF
      TL=TL+REPSIZ
      IF(RECORD(TDL).NE.MDATA) THEN
         NDTD=NDTD+1
         DTD(NDTD)=FLOAT(RECORD(TDL))/(10**SCALE(TDLOC))
      ENDIF
      TDL=TDL+REPSIZ
      IF(RECORD(ZL).NE.MDATA) THEN
         NDZ=NDZ+1
         DZ(NDZ)=FLOAT(RECORD(ZL))/(10**SCALE(ZLOC))
      ENDIF
      ZL=ZL+REPSIZ
      IF(RECORD(IDRL).NE.MDATA) THEN
         NDIR=NDIR+1
         DIR(NDIR)=FLOAT(RECORD(IDRL))/(10**SCALE(IDRLOC))
      ENDIF
      IDRL=IDRL+REPSIZ
      IF(RECORD(SPDL).NE.MDATA) THEN
         NDSP=NDSP+1
         DSP(NDSP)=FLOAT(RECORD(SPDL))/(10**SCALE(SPDLOC))
      ENDIF
      SPDL=SPDL+REPSIZ
70    CONTINUE
      NLEV=NDP
      CALL DDEST('NDP=',NDP)
      CALL DDEST('NDZ=',NDZ)
      CALL DDEST('NDT=',NDT)
      CALL DDEST('NDTD=',NDTD)
      CALL DDEST('NDIR=',NDIR)
      CALL DDEST('NDSP=',NDSP)
      ISTAT=0
      RETURN
      END
