#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include "gimfiles.h"
#include "gimparm.h"
#include "direct.h"
#include "gcmcidas.h"
#include "bandparams.h"
extern Direct_ptr dirp[MAXCMDOPT];
extern char **varg;
extern int bug;

/* MTSAT specific things */
#define MTSAT1_ID 84
#define MTSAT2_ID 85
#define CALIB_BLOCK_SIZE 6528*4
#define CALIB_NFILTERS 5

/* Define calibration structure */
struct CalibOffset {
  char Name[4];
  int  Offset;
};
struct CalibHeader {
  char SatName[8];
  char CoefType[8];
  struct CalibOffset Offset[CALIB_NFILTERS];
};

/* 17 April 2007 : JPDM
   Modified to add in MTSAT

   12 July 2010 : RIP
   Modify to recognize MTSAT2 satid (85)  as mtsat-type data
*/

int gimsetup (void)
{
 int  loc, len;
 int i, j, k;	/* index */
 int line, elem;
 int nbands;	/* number of bands to be put in area0 header */
 const int  mask1 = 0x00000001,
            mask2 = 0x00000002,
            mask3 = 0x00000004,
            mask4 = 0x00000008,
            mask5 = 0x00000010,
            mask6 = 0x00000020,
            mask7 = 0x00000040,
            mask8 = 0x00000080,
            mask9 = 0x00000100,
            mask10= 0x00000200,
            mask11= 0x00000400,
            mask12= 0x00000800;

 const int  mask[12] = {0x00000001, 0x00000002, 0x00000004, 0x00000008, 0x00000010, 0x00000020,
                        0x00000040, 0x00000080, 0x00000100, 0x00000200, 0x00000400, 0x00000800};
 char cerror[120];
#ifdef OLD_CODE
 int  codicils[128*7];
#else
 int  *pCodicils = NULL;
#endif

 struct CalibHeader CalibHdr;
 struct CalibHeader outCalibHdr;
 char inCalibData[CALIB_BLOCK_SIZE];
 char outCalibData[CALIB_BLOCK_SIZE];
 int in_offset = 0;
 int out_offset = 0;

 /*for (i=0; i<64; i++) { gf[0].dir[i] = gf[1].dir[i]; }*/ /* done in giminit */

 for (i=1; i<=num_in_files; i++)
  {
   if (dirp[0]->yyddd != dirp[i]->yyddd)
    {(void)printf("file time does not agree with base time\n");
     (void)printf("base time %6ld file time %6ld for file %2d %s\n",
      dirp[0]->yyddd, dirp[i]->yyddd, i, gf[i].file);
    }
   if (dirp[0]->time != dirp[i]->time)
    {(void)printf("file time does not agree with base time\n");
     (void)printf("base time %6ld file time %6ld for file %2d %s\n",
      dirp[0]->time, dirp[i]->time, i, gf[i].file);
    }
  }

 for (i=1; i<=num_in_files; i++)
  {
   if (dirp[i]->lcor < gp.top_lin_im) {gp.top_lin_im = dirp[i]->lcor;}
   if (dirp[i]->ecor < gp.lft_ele_im) {gp.lft_ele_im = dirp[i]->ecor;}
   line = dirp[i]->lcor + (dirp[i]->nlin - 1) * dirp[i]->lres;
   elem = dirp[i]->ecor + (dirp[i]->nele - 1) * dirp[i]->eres;
   if (line > gp.bot_lin_im) {gp.bot_lin_im = line;}
   if (elem > gp.rgt_ele_im) {gp.rgt_ele_im = elem;}
  }
 (void)printf("output area bounds\n");
 (void)printf("top left  line %6d element %6d\n", gp.top_lin_im, gp.lft_ele_im);
 (void)printf("bot right line %6d element %6d\n", gp.bot_lin_im, gp.rgt_ele_im);
 dirp[0]->lcor = gp.top_lin_im;
 dirp[0]->ecor = gp.lft_ele_im;
 dirp[0]->nlin = (gp.bot_lin_im - gp.top_lin_im) / dirp[0]->lres + 1;
 dirp[0]->nele = (gp.rgt_ele_im - gp.lft_ele_im) / dirp[0]->eres + 1;

 /* For each of the five channels find the first file that has it.  Use that file
    for that channel. */
 for (j=0; j<MAXBANDS; j++)
   {gp.chan_loc[j] = 0;
   for (i=1; i<=num_in_files; i++)
   {
    if ((dirp[i]->chan_map & mask[j]) != 0)
     { gp.chan_loc[j] = i; break; }
   }
  }
 printf("band locations: %2d %2d %2d %2d %2d %2d \n", gp.chan_loc[0], gp.chan_loc[1],
   gp.chan_loc[2], gp.chan_loc[3], gp.chan_loc[4], gp.chan_loc[5] );
 /* Also need relative position of band inside pixel for particular area */
 /* Get the number of bands in area0, the number of bands being united. */
 nbands = 0;
 for (j=0; j<MAXBANDS; j++) {if (gp.chan_loc[j] > 0) nbands++;}
 dirp[0]->nbands = nbands;

 /* Set the area0 channel map word to show the makeup of the new bands */
 dirp[0]->chan_map = 0;
 if (gp.chan_loc[0] > 0) dirp[0]->chan_map = dirp[0]->chan_map | mask1;
 if (gp.chan_loc[1] > 0) dirp[0]->chan_map = dirp[0]->chan_map | mask2;
 if (gp.chan_loc[2] > 0) dirp[0]->chan_map = dirp[0]->chan_map | mask3;
 if (gp.chan_loc[3] > 0) dirp[0]->chan_map = dirp[0]->chan_map | mask4;
 if (gp.chan_loc[4] > 0) dirp[0]->chan_map = dirp[0]->chan_map | mask5;
 if (gp.chan_loc[5] > 0) dirp[0]->chan_map = dirp[0]->chan_map | mask6;
 if (gp.chan_loc[6] > 0) dirp[0]->chan_map = dirp[0]->chan_map | mask7;
 if (gp.chan_loc[7] > 0) dirp[0]->chan_map = dirp[0]->chan_map | mask8;
 if (gp.chan_loc[8] > 0) dirp[0]->chan_map = dirp[0]->chan_map | mask9;
 if (gp.chan_loc[9] > 0) dirp[0]->chan_map = dirp[0]->chan_map | mask10;
 if (gp.chan_loc[10] > 0) dirp[0]->chan_map = dirp[0]->chan_map | mask11;
 if (gp.chan_loc[11] > 0) dirp[0]->chan_map = dirp[0]->chan_map | mask12;
 printf("area0 channel map %ld\n", dirp[0]->chan_map);

 /* Check the length of the area0 level section */
 if (dirp[0]->len_lev < 8)
  {dirp[0]->line_pref += (8-dirp[0]->len_lev); 	/* Add incriment of len_lev */
   dirp[0]->len_lev = 8;}
 if (dirp[0]->validity_code != 0)
  {dirp[0]->line_pref = 4;}
  else
  {dirp[0]->line_pref = 0;}
 dirp[0]->line_pref = dirp[0]->line_pref + dirp[0]->len_doc +
   dirp[0]->len_cal + dirp[0]->len_lev;

 /* Now make area0.  This defines area0 io routine parameters. */
 i = makara (gf[0].fileno, (int  *)dirp[0]);
 if (i != 0) {(void)printf("gimsetup: makara failed\n"); return -1;}

 /* gp.lev_flag tells whether there is a level section in the prefix for each
    file.  If present, lev_flag is false (zero). */
 /* gp.lev_loc is used only if there is a level section in the prefix. */
 /* If there is a level section then the condition of a certian band missing
    in a particular line must be delt with. */
 for (i=1; i<=num_in_files; i++)
  {
   if (dirp[i]->len_lev != 0)
    {
     gp.lev_flag[i] = 0;
     if (dirp[i]->validity_code != 0)
      {gp.lev_loc[i] = 4;}
      else
      {gp.lev_loc[i] = 0;}
     gp.lev_loc[i] = gp.lev_loc[i] + dirp[i]->len_doc + dirp[i]->len_cal;
    }
   else
    {
     /* Area should be single band */
     gp.lev_loc[i] = 0;
     gp.lev_flag[i] = 1;
    }
  }
 if (dirp[0]->validity_code != 0)
  {gp.lev_loc[0] = 4;}
  else
  {gp.lev_loc[0] = 0;}
 gp.lev_loc[0] = gp.lev_loc[0] + dirp[0]->len_doc + dirp[0]->len_cal;
 if (bug > 0)
  {(void)printf("level location: %2d %2d %2d %2d %2d %2d %2d\n", gp.lev_loc[0],
    gp.lev_loc[1], gp.lev_loc[2], gp.lev_loc[3], gp.lev_loc[4], gp.lev_loc[5], gp.lev_loc[6]);
   (void)printf("level flag:     %2d %2d %2d %2d %2d %2d %2d\n", gp.lev_flag[0],
    gp.lev_flag[1],gp.lev_flag[2],gp.lev_flag[3],gp.lev_flag[4],gp.lev_flag[5], gp.lev_loc[6]);
  }

 /* In each file for each band, the band's location relative to other bands
    present may vary from line to line.  However, if this is a single band file
    then the offset is always 0 so set it here.  Else set to -1 and find it when
    each line is read in. */
 for (i=0; i<MAXBANDS; i++)
  {if (gp.lev_flag[gp.chan_loc[i]] == 1)
    {gp.lev_off[i] = 0;}
    else
    {gp.lev_off[i] = -1;}
  }
 if (bug > 0)
  {(void)printf("level offset:   %2d %2d %2d %2d %2d %2d by channel\n",
   gp.lev_off[0], gp.lev_off[1], gp.lev_off[2], gp.lev_off[3], gp.lev_off[4], gp.lev_off[5]);}

 /* Now get and copy calibration */
 if( MTSAT1_ID == dirp[1]->sat_id   ||   MTSAT2_ID == dirp[1]->sat_id ){
   /* For MTSAT */
   /* Transfer navigation and calibration codicils from area1 to area0. */
   /* Read in from first input file */
   loc = dirp[1]->nav_loc;
   errno = 0;
   i = lseek(gf[1].fileno,loc,0);
   if (i < 0)
     {sprintf(cerror,"%s: gimsetup: lseek error l1 errno= %d fdi: %d %ld %d\n",
	      varg[0],errno,gf[1].fileno,loc,i);
     perror(cerror); (void)printf(cerror); return -1;}
   
   /* Copy navigation block */
   len = dirp[1]->cal_loc - dirp[1]->nav_loc;
   
   /* Allocate memory for navigation */
   if( NULL == (pCodicils = (int  *) calloc(len,sizeof(int))) ){
     {sprintf(cerror,"%s: gimsetup: Allocating codicils\n",
	      varg[0]);
     perror(cerror); (void)printf(cerror); return -1;}
   }
   errno = 0;
   i = read (gf[1].fileno, pCodicils, len);
   if (i != len && errno != 0)
     {sprintf(cerror,"%s: gimsetup: read error r1 errno= %d fd: %d "
	      " %ld %d %ld\n",varg[0],errno,gf[1].fileno,len,i,loc);
     (void)printf(cerror); perror(cerror); return -1;}
   /* Write to output file, area0 */
   loc = dirp[0]->nav_loc;
   errno = 0;
   i = lseek(gf[0].fileno,loc,0);
   if (i < 0)
     {sprintf(cerror,"%s: gimsetup: lseek error l1 errno= %d fdi: %d %ld %d\n",
	      varg[0],errno,gf[0].fileno,loc,i);
     perror(cerror); (void)printf(cerror); return -1;}
   
   len = dirp[0]->cal_loc - dirp[1]->nav_loc;
   errno = 0;
   i = write (gf[0].fileno, pCodicils, len);
   if (i != len && errno != 0)
     {sprintf(cerror,"%s: gimsetup: write error r1 errno= %d fd: %d "
	      " %ld %d %ld\n",varg[0],errno,gf[0].fileno,len,i,loc);
     (void)printf(cerror); perror(cerror); return -1;}
   
   free(pCodicils);

   /* Now loop round for other files to get to correct place in file */
   for(i=2;i<=num_in_files;i++){
     /* Read in from first input file */
     loc = dirp[i]->nav_loc;
     errno = 0;
     k = lseek(gf[i].fileno,loc,0);
     if (k < 0)
       {sprintf(cerror,"%s: gimsetup: lseek error l1 errno= %d fdi: %d %ld %d\n",
		varg[0],errno,gf[1].fileno,loc,k);
       perror(cerror); (void)printf(cerror); return -1;}
     
     /* Copy navigation block */
     len = dirp[i]->cal_loc - dirp[i]->nav_loc;
   
     /* Allocate memory for navigation */
     if( NULL == (pCodicils = (int  *) calloc(len,sizeof(int))) ){
       {sprintf(cerror,"%s: gimsetup: Allocating codicils\n",
		varg[0]);
       perror(cerror); (void)printf(cerror); return -1;}
     }
     errno = 0;
     k = read (gf[i].fileno, pCodicils, len);
     if (k != len && errno != 0)
       {sprintf(cerror,"%s: gimsetup: read error r1 errno= %d fd: %d "
		" %ld %d %ld\n",varg[0],errno,gf[1].fileno,len,k,loc);
     (void)printf(cerror); perror(cerror); return -1;}
     free(pCodicils);
   }

   /* Now copy calibration correctly */
   /* make output calibration block */
   errno = 0;   
   i = read (gf[1].fileno, &inCalibData[0], CALIB_BLOCK_SIZE);
   if (i != len && errno != 0)
     {sprintf(cerror,"%s: gimsetup: read error r1 errno= %d fd: %d "
	      " %ld %d %ld\n",varg[0],errno,gf[1].fileno,len,i,loc);
     (void)printf(cerror); perror(cerror); return -1;}
   /* Get header */
   memcpy(&CalibHdr,&inCalibData[0],sizeof(struct CalibHeader));

   /* Initialise calib block */
   memset(&outCalibData[0],'\0',CALIB_BLOCK_SIZE);

   /* Copy first file data into output */
   memcpy(&outCalibHdr,&CalibHdr,sizeof(struct CalibHeader));

   /* Setup output offsets */
   for(i=0;i<CALIB_NFILTERS;i++)
     outCalibHdr.Offset[i].Offset = 768+i*1024*sizeof(int);

   /* Copy header to output */
   memcpy(&outCalibData[0],&outCalibHdr,sizeof(struct CalibHeader));
   
   /* For first file loop round allowed bands */
   for(j=0;j<MAXBANDS;j++){
     /* If this file has band j */
     if( (dirp[1]->chan_map & mask[j]) != 0 ){
       /* Get and copy to correct output position the calibration data */
       in_offset = CalibHdr.Offset[j].Offset;
       out_offset = outCalibHdr.Offset[j].Offset;
       memcpy(&outCalibData[out_offset],&inCalibData[in_offset],
	      1024*sizeof(int));
     }
   }

   /* Now loop round rest of files */
   for(i=2;i<=num_in_files;i++){
     if( MTSAT1_ID != dirp[1]->sat_id   &&   MTSAT2_ID != dirp[1]->sat_id )
       {sprintf(cerror,"%s: gimsetup: SAT ID incorrect - must be for MTSAT\n",
		varg[0]);(void)printf(cerror); perror(cerror); return -1;}
     /* Read in new data */
     errno = 0;   
     k = read (gf[i].fileno, &inCalibData[0], CALIB_BLOCK_SIZE);
     if (k != len && errno != 0)
       {sprintf(cerror,"%s: gimsetup: read error r1 errno= %d fd: %d "
		" %ld %d %ld\n",varg[0],errno,gf[i].fileno,len,k,loc);
       (void)printf(cerror); perror(cerror); return -1;}
     /* Get header */
     memcpy(&CalibHdr,&inCalibData[0],sizeof(struct CalibHeader));
     /* Work out which band and copy 1024 values to new offset */
     /* Loop round allowed bands */
     for(j=0;j<MAXBANDS;j++){
       /* If this file has band j */
       if( (dirp[i]->chan_map & mask[j]) != 0 ){
	 /* Get and copy to correct output position the calibration data */
	 in_offset = CalibHdr.Offset[j].Offset;
	 out_offset = outCalibHdr.Offset[j].Offset;
	 memcpy(&outCalibData[out_offset],&inCalibData[in_offset],
		1024*sizeof(int));
       }
     }
   }

   /* Write calibration to output file */
   i = write (gf[0].fileno, &outCalibData, CALIB_BLOCK_SIZE);
   if (i != len && errno != 0)
     {sprintf(cerror,"%s: gimsetup: write error r1 errno= %d fd: %d "
	      " %ld %d %ld\n",varg[0],errno,gf[0].fileno,len,i,loc);
     (void)printf(cerror); perror(cerror); return -1;}
   

 } else {

   /* Transfer navigation and calibration codicils from area1 to area0. */
   /* Read in from first input file */
   loc = dirp[1]->nav_loc;
   errno = 0;
   i = lseek(gf[1].fileno,loc,0);
   if (i < 0)
     {sprintf(cerror,"%s: gimsetup: lseek error l1 errno= %d fdi: %d %ld %d\n",
	      varg[0],errno,gf[1].fileno,loc,i);
     perror(cerror); (void)printf(cerror); return -1;}
   
   /* Copy navigation block */
   len = dirp[1]->cal_loc - dirp[1]->nav_loc;
   
   /* Allocate memory for navigation */
   if( NULL == (pCodicils = (int  *) calloc(len,sizeof(int))) ){
     {sprintf(cerror,"%s: gimsetup: Allocating codicils\n",
	      varg[0]);
     perror(cerror); (void)printf(cerror); return -1;}
   }
   errno = 0;
   i = read (gf[1].fileno, pCodicils, len);
   if (i != len && errno != 0)
     {sprintf(cerror,"%s: gimsetup: read error r1 errno= %d fd: %d "
	      " %ld %d %ld\n",varg[0],errno,gf[1].fileno,len,i,loc);
     (void)printf(cerror); perror(cerror); return -1;}
   /* Write to output file, area0 */
   loc = dirp[0]->nav_loc;
   errno = 0;
   i = lseek(gf[0].fileno,loc,0);
   if (i < 0)
     {sprintf(cerror,"%s: gimsetup: lseek error l1 errno= %d fdi: %d %ld %d\n",
	      varg[0],errno,gf[0].fileno,loc,i);
     perror(cerror); (void)printf(cerror); return -1;}
   
   len = dirp[0]->cal_loc - dirp[1]->nav_loc;
   errno = 0;
   i = write (gf[0].fileno, pCodicils, len);
   if (i != len && errno != 0)
     {sprintf(cerror,"%s: gimsetup: write error r1 errno= %d fd: %d "
	      " %ld %d %ld\n",varg[0],errno,gf[0].fileno,len,i,loc);
     (void)printf(cerror); perror(cerror); return -1;}
   
   free(pCodicils);

   /* Simple copy of goes calibration */
   
   /* Copy navigation block */
   len = dirp[1]->data_loc - dirp[1]->cal_loc;

   /* Allocate memory for navigation */
   if( NULL == (pCodicils = (int  *) calloc(len,sizeof(int))) ){
     {sprintf(cerror,"%s: gimsetup: Allocating codicils\n",
	      varg[0]);
     perror(cerror); (void)printf(cerror); return -1;}
   }
   errno = 0;
   i = read (gf[1].fileno, pCodicils, len);
   if (i != len && errno != 0)
     {sprintf(cerror,"%s: gimsetup: read error r1 errno= %d fd: %d "
	      " %ld %d %ld\n",varg[0],errno,gf[1].fileno,len,i,loc);
     (void)printf(cerror); perror(cerror); return -1;}
   /* Write to output file, area0 */
   loc = dirp[0]->cal_loc;
   errno = 0;
   i = lseek(gf[0].fileno,loc,0);
   if (i < 0)
     {sprintf(cerror,"%s: gimsetup: lseek error l1 errno= %d fdi: %d %ld %d\n",
	      varg[0],errno,gf[0].fileno,loc,i);
     perror(cerror); (void)printf(cerror); return -1;}
   
   len = dirp[0]->data_loc - dirp[1]->cal_loc;
   errno = 0;
   i = write (gf[0].fileno, pCodicils, len);
   if (i != len && errno != 0)
     {sprintf(cerror,"%s: gimsetup: write error r1 errno= %d fd: %d "
	      " %ld %d %ld\n",varg[0],errno,gf[0].fileno,len,i,loc);
     (void)printf(cerror); perror(cerror); return -1;}
   
   free(pCodicils);
 }
 
 return 0;
}
